// ==UserScript==
// @name         ADS changeset comment search
// @version      0.4
// @description  Places a searchbox somewhere to search commit messages
// @author       Tobias Sachs
// @match        https://ads/*
// @grant        GM_addStyle
// @updateURL    https://src.twobees.de/?p=tampermonkeyscripts.git;a=blob_plain;f=AzureDevOpsChangesetCommentSearch.js;hb=HEAD
// @downloadURL  https://src.twobees.de/?p=tampermonkeyscripts.git;a=blob_plain;f=AzureDevOpsChangesetCommentSearch.js;hb=HEAD
// ==/UserScript==

// 0.4: - improve position of search box
// 0.3: - fix abort search
//      - filter reuslts by committers name
//      - no need for an observer
//      - use GM_addStyle to improve readability
//      - "Escape" stops search and removes resultstab

/* jshint esversion:6 */
(function() {
    'use strict';

    let searchDelayTimerId;
    let pollUrl = 'https://ads/HeBa/Entwicklung/_apis/tfvc/changesets?maxCommentLength=500?$top=1000?searchCriteria.itemPath=$/Entwicklung/HEAD';
    let skipParm = '&$skip=';
    let changeSetUrl = `/HeBa/Entwicklung/_versionControl/changeset/`;

    let totalReceived = 0;
    let matchesFound = 0;
    let searchTerm = "";

    GM_addStyle(`#ts_cs_search {
                    position: absolute;
                    left: 50%;
                    top: 0px;
                    height: auto;
                    z-index: 9999;
                    transform: translateX(-50%);
                 }`);
    GM_addStyle(`.ts_match {
                     background: orange;
                 }`);
    GM_addStyle(`.ts_found {
                     padding: 2px;
                 }`);
    GM_addStyle(`.ts_found a {
                     text-decoration: none;
                 }`);
    GM_addStyle(`#ts_searchResults{
                     max-height: 500px;
                     background:#CCCCCC;
                     overflow: auto;
                 }`);

    let searchDelayed = function(){
        if (searchDelayTimerId) {
            clearTimeout(searchDelayTimerId);
        }

        searchDelayTimerId = setTimeout(
            () => {
                searchDelayTimerId = undefined;
                startSearching();
            },
            200);
    };

    let pressEscapeToAbort = function(e) {
        if (e.key == "Escape"){
            document.getElementById('ts_searchBox').value = "";
            startSearching(); // will abort search and clean results
        }
    };

    let requestNext = function(skipCnt, currentSearch){
        let xhr = new XMLHttpRequest();
        xhr.onload = () => searchItems(xhr, currentSearch);
        xhr.onerror = function (e){
            console.warning(`poll error: ${e.type}: ${e.loaded} bytes transferred\n` + JSON.stringify(e));
        };
        xhr.open('GET', pollUrl + skipParm + skipCnt, true);
        xhr.send();
    };

    let searchItems = function(xhr, searchStr){
        if (xhr.status !== 200){
            console.info("poll failed: " + xhr.statusText);
            return;
        }

        if (searchStr !== searchTerm)
        {
            console.info("Searchterm changed while searching to " + searchTerm);
            return;
        }

        let sp = searchStr.split(/(von:|from:)/i);
        let searchName = undefined;
        let search = searchStr;
        if (sp.length == 3){
            search = sp[0].trim();
            searchName = sp[2].trim();
        }

        let resultDiv = document.getElementById('ts_searchResults');
        let statsSpan = document.getElementById('ts_searchStats');

        var d = JSON.parse(xhr.responseText);
        let regex = undefined;
        let regexAuthor = undefined;
        if (search){
            regex = new RegExp("("+search+")", 'gi');
        }
        if (searchName){
            regexAuthor = new RegExp("("+searchName+")", 'i');
        }
        totalReceived += d.value.length;
        for (let e in d.value){
            let cs = d.value[e];
            if (regex !== undefined && regexAuthor !== undefined)
            {
                if (!(cs.comment && regex.test(cs.comment) && regexAuthor.test(cs.author.displayName)))
                {
                    continue;
                }
            }
            else
            {
                if ((
                       (cs.comment && regex !== undefined && regex.test(cs.comment))
                    || (regexAuthor !== undefined && regexAuthor.test(cs.author.displayName))
                    ) == false){
                    continue;
                }
            }
            matchesFound++;
            let comment;
            let author;
            if (regex){
                comment = cs.comment.replace(regex, `<span class="ts_match">$1</span>`);
            }
            else
            {
                comment = cs.comment;
            }
            if (regexAuthor){
                author = cs.author.displayName.replace(regexAuthor, `<span class="ts_match">$1</span>`);
            }
            else{
                author = cs.author.displayName;
            }


            let item = `<div class='ts_found'>
                               <a href='${changeSetUrl}${cs.changesetId}'>
                                  ${cs.changesetId}: ${comment}  (${author})
                               </a>
                            </div>`;
            resultDiv.innerHTML += item;
    }
    statsSpan.innerHTML = "matches: " + matchesFound +" searched comments: " + totalReceived;
    if (d.value.length <= 0)
    {
        statsSpan.innerHTML += " -- search done.";
        return;
    }

    requestNext(totalReceived, searchStr);
};

 let startSearching = function (){
    searchTerm = document.getElementById('ts_searchBox').value;
    let results = document.getElementById('ts_searchResults');
    totalReceived = 0;
    matchesFound = 0;
    results.innerHTML = "";
    document.getElementById('ts_searchStats').innerHTML = "";

    if (searchTerm.length < 3){ return; }

    console.debug("new search term: '" + searchTerm+"'");
    requestNext(0, searchTerm);
};

let addSearch = () => {
    if (window.location.href.includes("_apis")){ return; }

    let search = document.getElementById('ts_cs_search');
    if (search){ return; }

    console.debug("adding searchbox");
    let html = `<div id="ts_cs_search">
                        <input id="ts_searchBox" placeholder="searchstring [von:Name]" />
                        <span id="ts_searchStats"></span>
                        <div id="ts_searchResults"></div>
                    </div>`;

        document.body.insertAdjacentHTML('afterbegin', html);
        let s = document.getElementById('ts_searchBox');
        s.addEventListener("keyup", searchDelayed);
        document.body.addEventListener("keyup", pressEscapeToAbort);

    };

addSearch();
})();
